function E=convex_iter(A,b,c,K,varargin)
% function E=convex_iter(A,b,c,K,varargin)
% 
% Solve an SDP (or mixed LP/SDP) and attempt to reduce the
% dimensionality of the result by Dattorro's convex iteration
% technique.


maxrank=3;
swgt=100;
show='show';
maxiter= 20; %K.s;
eigratio = 5e-2;
zerotol  = 1e-5;
fid = 0;

pars.fid=0;
pars.printlevel = 0;
pars.fastmode=1;
pars.affine = 1;
pars.solver='csdp';
pars.eps = 1e-08;
pars.axtol  = 1e-2;
pars.atytol = 1e-2;
pars.objtol = 1e-2;


i=1;
while i<=length(varargin)
  if isstr(varargin{i})
    switch varargin{i}
      case 'swgt',                        % Weight on search direction
        swgt=varargin{i+1};i=i+1;
      case 'maxrank',                     % Desired rank
        maxrank=varargin{i+1};i=i+1;
      case 'eigratio'
        eigratio=varargin{i+1};i=i+1;
      case 'winit',                       % Initial search direction guess
        winit=varargin{i+1};i=i+1;
      case 'useinit', useinit=1;          % Whether to use last point as initial
      case 'solver',                      % select the solver
        solver=varargin{i+1};i=i+1;
      case 'maxiter',                     % Set maximum iterations
        maxiter=varargin{i+1};i=i+1; 
      case 'pars'
        pp = varargin{i+1}; i=i+1;
        flds=fieldnames(pp);
        for j=1:length(flds)
          pars.(flds{j}) = pp.(flds{j});
        end
    end
  end
  i=i+1;
end

i=1;
n  = K.s;
nn = n*n;
cnrm=norm(c);
sip{i}=Inf;
dinfo{i}.cpusec=0;
bestsip=Inf;

% Process the initial point if needed
W{i}=sp0(nn,1);
if exist('winit')
  if length(vec(winit))==nn
    W{i}=vec(winit);
  else
    fprintf('**Warning: winit ignored due to dimension problems\n');
  end
end
fprintf(fid,'axtol %.2e  atytol: %.2e  objtol: %.2e  fastmode: %d\n',...
        pars.axtol, pars.atytol, pars.objtol, pars.fastmode)
fprintf(fid,'K.l: %d  K.s: %d  swgt: %.2e\n',...
        K.l, K.s, swgt);
fprintf(fid,'Starting Iteration, maxiters: %d  eigratio: %.2e\n',...
        maxiter, eigratio);

fprintf(fid,'%4s %9s %10s %10s %8s %3s %8s\n',...
        'Iter','c*xv','e4to3','SIP','Scode','Aff','Minutes');

for i=1:maxiter
  ww = [sp0(K.l,1); W{i}];
  wnorm = norm(ww);
  if wnorm>0, ww = ww / norm(ww); end
  cpred = c + cnrm*swgt*ww;             % New objective

  % Make prediction
  [xH{i} yH{i} hinfo{i}]=sdpsolver(A,b,cpred,K,pars);
  [xLh xSh]=separate(xH{i},K);
  [qq ll]=eig(mat(xSh));
  l=diag(ll);
  e4to3=sum(l(1:(n-maxrank)))/l(n-maxrank+1);
  rnk=sum(l>1e-5);

  fprintf(fid,'%3d: %9.2e %10.3e %10.3e %8s %3d %8.2f\n',...
          i, c'*xH{i}, e4to3, sip{i}, hinfo{i}.msg,...
          pars.affine, hinfo{i}.cpusec/60);
  if e4to3 <= eigratio || rnk <= maxrank 
    break;
  end

  % Determine next search direction, compute search inner product (sip)
  % New version, no minimization required, use eigvals
  if i<maxiter
    tic;
    l2=zeros(n,1);
    if i==1
      l2(1:n-maxrank)=(1+abs(l(1:n-maxrank)));
      snrm = sum(l2);
    else
      l2(1:n-maxrank)=1;
      l2 = l2 / (sum(l2)) * snrm;
    end
    tmp = qq*diag(l2)*qq';
    W{i+1} = vec((tmp+tmp')/2);
    L{i+1}=l; L2{i+1} = l2;
    sip{i+1}=l' * l2;
    dinfo{i+1}.cpusec=toc;
    bestsip=sip{i+1};
  end
end

if e4to3 <= eigratio || rnk <= maxrank
  converged='YES';
% elseif stalls >= maxstalls
%   converged='Stalled';
elseif i >= maxiter
  converged='Maxiter';
end
fprintf(fid,'Converged: %s\n',converged);

last=i;
xh=xH{last};
[xLh xSh]=separate(xh,K);

for i=1:last
  times(i,:)=[hinfo{i}.cpusec dinfo{i}.cpusec];
end
totaltime=sum(sum(times));

clear i j;
env2struct('E',who());
