function [nois obj]=get_noise(I, cln, type, rinit, frac)
% function [nois corel ip]=get_noise(I, cln, type, rinit, frac)
% 
% Add noise to an objective or distance matrix depending on 'type'.
% Initialize random number generator to 'rinit' for reproducibility.
% Add 'frac' amount of noise and return the noised object along with
% its correlation, normalized difference, and normalized inner product
% to the original.

rand('state',rinit);
randn('state',rinit);

switch type
  case 'obj'
    n=length(cln);
    noi=randn(n,1);
    nois = cln/norm(cln) * (1-frac) + noi/norm(noi) * frac;
    obj = cln;

  case 'dmat'
    D = gram2distmat(mat(I.xS));
    [m n]=size(D);
    noi = triu(randn(m,m),1);
    noi = noi + noi';
    noi = noi / norm(noi,'fro');
    nrm = norm(D,'fro');
    Dn = abs((1-frac)*D + frac*nrm*noi);

    Dn=triu(Dn);                          % Set equalities
    Dn(I.eqidx)=I.eqdist;
    idx=find(Dn(I.inidx)<I.indist);       % Check for distances that are too small
    Dn(I.inidx(idx)) = I.indist(idx);
    Dn = Dn+Dn';

    nois = Dn;
    obj = D;

  case 'both'
    nois.Dn = get_noise(I,cln,'dmat',rinit,frac);
    nois.cnoi = get_noise(I,cln,'obj',rinit,frac);
    obj = cln;

  otherwise
    fprintf('Bad noise type "%s"\n',type);
end
