function E=nearest_edm2(xLh,I,varargin)
% function [G info]=nearest_edm(D,I)
%
% Find the closest EDM to distances given in xLh.  Equality and
% inequality constraints in I are included in the problem.  This code
% uses active_set() to try to handle the inequality constraints as
% efficiently as possible.

thresh = max(xLh);

i=1;
while i<=length(varargin)
  if isstr(varargin{i})
    switch varargin{i}
      case 'thresh', thresh=varargin{i+1}; i=i+1;
      case 'mask',   mask=varargin{i+1}; i=i+1;
    end
  end
  i=i+1;
end

if ~exist('mask')
  mask = (xLh <= thresh);
end

ndist = length(find(mask));
[Aeq Ain beq bin] = separate_eq(I);

% Equality section, both original equality and distance targets
[tmp ASeq] = separate(Aeq,I.K);
[tmp ASdist] = separate(Ain(mask,:),I.K);         
bdist = xLh(mask); % + bin(mask);

% Inequality sectio of the program
% [tmp ASin] = separate(Ain(~mask,:),I.K);          
% bin = bin(~mask,:);
[tmp ASin] = separate(Ain,I.K);
bin = bin;

A = [sp0(size(ASeq,1),2*ndist)  ASeq
     spI(ndist)  -spI(ndist)    ASdist
     sp0(size(ASin,1),2*ndist)  ASin
];
b = [beq; bdist; bin];
c = [ones(2*ndist,1); sp0(I.K.s^2,1)];

K.l = ndist*2;
K.s = I.K.s;

iseq = [ones(size(ASeq,1),1); 
        ones(size(ASdist,1),1); 
        zeros(size(ASin,1),1);
];
iseq = logical(iseq);

[x err Es]=active_set(A,b,c,K,iseq,varargin{:},'maxiter',10);

xS = x(2*ndist+1:length(x));

E.x = xS;
E.err = err;
E.xdist = x;
E.Es = Es;
E.totaltime = Es.totaltime;
