#!/bin/bash
#PBS -j oe
# 
# This is a script which was useful for large scale testing of our
# geometry methods.  Typically we used 'sed' to substitute the
#following parameters
# 
# PROTKEY to the protein of interest
# METH to domo, nedm, or other methods 
# LVL to the level of noise desired
# RINIT for the random number generator initialization, changes perturbation
# 

base=$HOME/data/4/coord-bench
datfile=$base/dats/PROTKEY.dat
objfile=$base/objectives/PROTKEY.cL

hostname=`hostname`;
jobid=$PBS_JOBID

workdir=$HOME/jediwork/coord-bench/jobs/METH/running
mkdir -p $workdir
cd $workdir
logfile=$workdir/PROTKEY.METH.LVL.RINIT.log

dest=$HOME/jediwork/coord-bench/METH
mkdir -p $dest
savefile=$dest/PROTKEY.METH.LVL.RINIT.mat

matlab='/soft/matlab7.2/bin/matlab  -nosplash -nojvm -nodisplay' 
$matlab -logfile $logfile <<EOF | sed 's/>>\s*//g'
!echo PROTKEY METH LVL RINIT $hostname $jobid
!date
%addpath ~/code/sdp-folding/stable

lvl = LVL;
rinit = RINIT;
protkey = 'PROTKEY';
meth = 'METH';

I = instanceR('$datfile','full');
cL = load('$objfile');
[nois obj] = get_noise(I, cL, 'MTYPE', rinit, lvl);
E = METH_predict(I,nois,'frac',0.30);
objn = E.obj;
[corel ip nrm] = obj_sim(obj, objn);
G = mat(E.x);
[x3 rms] = gram2struct(G,'I',I,'srmsd','noshow');
[xe r3err] = rankr_approx(I.A,I.b,I.K,G);

!date

stats = {...
  'P0>','P1>', ...
  'protkey', protkey, ...
  'meth', meth, ...
  'host','$hostname', ...
  'lvl',lvl, ...
  'rmsd',rms, ...
  'r3err',r3err, ...
  'Minutes',E.totaltime/60, ...
  'corr', corel, ...
  'iprod',ip, ...
  'nrm', nrm, ...
  'nb',I.K.s, ...
  'nr',I.nr, ...
  'rinit',rinit, ...
  '','' ...
};
fprintf('\n');
for i=1:2:length(stats)
  fprintf('%12s ',stats{i});
end
fprintf('\n');
for i=2:2:length(stats)
  if isstr(stats{i})
    fprintf('%12s ',stats{i});
  else
    fprintf('%12g ',stats{i});
  end
end
fprintf('\n');
if isfield(E,'data')
  data = E.data;
else
  data = {};
end

clear E I xe cL obj noi;
save $savefile

EOF
